# ウェクスラー成人知能尺度と認知症

library(dplyr)

# データ
data=data.frame(WAIS=c(4:20),n=c(2,1,2,3,2,6,6,6,2,6,7,3,4,1,1,1,1),Y=c(1,1,1,2,2,2,1,1,0,1,2,0,0,0,0,0,0)) %>% mutate(p=Y/n)

# 特徴量行列
X=as.matrix(cbind(rep(1,nrow(data)),data[,colnames(data) %in% c("WAIS")]))

# 各変数
pis=data$p;n=data$n;y=data$Y

# 回帰係数の初期値
alpha=0.05;beta=0.05

# 反復計算回数
ite=100

# 回帰係数の更新歴の保存データ
alpha_beta_data=data.frame(ite=1:ite,alpha=0,beta=0)

for(j in 1:ite){
  # 予測確率
  p=1/(1+exp(-(X%*%c(alpha,beta))))

  # Wの逆行列
  V=array(0,dim=c(nrow(data),nrow(data)))
  diag(V)=c(n*p*(1-p))

  # 回帰係数の更新
  mat=solve(t(X)%*%V%*%(X))%*%t(X)%*%(y-n*p)
  alpha=alpha+mat[1];beta=beta+mat[2]
  print(alpha);print(beta)
  alpha_beta_data$alpha[j]=alpha
  alpha_beta_data$beta[j]=beta
}

# 予測確率の保存
data=data %>% dplyr::mutate(predict=1/(1+exp(-(X%*%c(alpha,beta)))))

# 予測値の保存
data=data%>%dplyr::mutate(mu=n*predict)

# 目的変量との比較（plot）
plot(data$WAIS,data$p,type="p",col=2,xlim=c(min(data$WAIS),max(data$WAIS)),ylim=c(min(data$p),max(data$p)),xlab="WAIS",ylab="p",pch=1)

par(new=T)

plot(data$WAIS,data$predict,type="p",col=3,xlim=c(min(data$WAIS),max(data$WAIS)),ylim=c(min(data$p),max(data$p)),xlab="WAIS",ylab="p",pch=16)

mu=data$mu

# 残差にあたる部分（尤離度）
D_y_mu1=y*log(y/mu);D_y_mu2=(n-y)*log((n-y)/(n-mu))
D_y_mu1[is.nan(D_y_mu1)>0]=0;D_y_mu2[is.nan(D_y_mu2)>0]=0
D_y_mu=2*D_y_mu1+2*D_y_mu2

# 回帰にあたる部分（尤離度）
D_y_y1=y*log(y/mean(y));D_y_y2=(n-y)*log((n-y)/(n-mean(y)))
D_y_y1[is.nan(D_y_y1)>0]=0;D_y_y2[is.nan(D_y_y2)>0]=0
D_y_y=2*D_y_y1+2*D_y_y2

sj=sqrt(diag(solve(t(X)%*%V%*%(X))))
V2=V;diag(V2)=sqrt(diag(V2))
Xs=V2%*%X
Hs=Xs%*%solve(t(Xs)%*%Xs)%*%t(Xs)

# 尤離度決定係数
R_dev=1-sum(D_y_mu)/sum(D_y_y)

# H0：回帰係数=0の統計量
Z=c(alpha,beta)/sj

# 尤離度
d=D_y_mu

# ピアソン残差
r_pi=(y-mu)/sqrt(n*p*(1-p))

# 尤離度残差
r_di=sign(y-mu)*sqrt(d)

# 標準ピアソン残差
r_pi/sqrt(1-diag(Hs))

# 標準尤離度残差
r_di/sqrt(1-diag(Hs))
