# 指数分布回帰モデル

library(dplyr)

# 冠状動脈性心疾患と喫煙のデータで解析

# 死亡数
DEATH=c(32,104,206,186,102,2,12,28,28,31)

# 観測者数
PERSON=c(52407,43248,28612,12663,5317,18790,10673,5710,2585,1462)

X2=c(rep(1,5),rep(0,5))
X3=c(1:5,1:5)
X4=X3^2
X5=c(1:5,rep(0,5))

# 特徴量行列
X=cbind(rep(1,length(X3)),X2,X3,X4,X5)

# 目的変量
pis=DEATH

# 初期値
alpha=rep(0.01,(ncol(X)))

# 反復計算回数と学習率
ite=1000;eta=0.01

# パラメータを保存するデータ
alpha_beta_data=array(0,dim=c(ite,ncol(X)))

for(j in 1:ite){
  V=array(0,dim=c(nrow(X),nrow(X)))
  diag(V)=1
  D=diag(c(1/exp(X%*%alpha)))
  mat=solve(t(X)%*%V%*%(X))%*%t(X)%*%V%*%D%*%(pis-exp(X%*%alpha))
  alpha=alpha+eta*mat
  alpha_beta_data[j,]=alpha
  loglik=sum(-X%*%alpha-pis*exp(-X%*%alpha))
  print(paste0("対数尤度:",loglik))
}

# 予測値
predict=exp(X%*%alpha)
data=data.frame(X,pis,predict)

plot(c(1:nrow(data)),data$pis,type="p",col=2,xlim=c(1,nrow(data)),ylim=c(min(c(pis,predict)),max(c(pis,predict))),xlab="samples",ylab="predict",main="指数回帰model",pch=16)

par(new=T)

plot(c(1:nrow(data)),data$predict,type="p",col=3,xlim=c(1,nrow(data)),ylim=c(min(c(pis,predict)),max(c(pis,predict))),xlab="samples",ylab="predict",pch=1)

# 尤離度決定係数
R2_dev=1-sum(log(predict/pis)+(pis-predict)/predict)/sum(log(mean(pis)/pis)+(pis-mean(pis))/mean(pis))
