# 逆ガウス分布回帰モデル

# 死亡者数
DEATH=c(32,104,206,186,102,2,12,28,28,31)

# 観測者数
PERSON=c(52407,43248,28612,12663,5317,18790,10673,5710,2585,1462)

X2=c(rep(1,5),rep(0,5))
X3=c(1:5,1:5)
X4=X3^2
X5=c(1:5,rep(0,5))

# 特徴量行列
X=cbind(rep(1,length(X3)),X2,X3,X4,X5)

# 目的変量
pis=DEATH

# パラメータの初期値
alpha=rep(0.001,(ncol(X)))

# 反復計算回数、パラメータ、学習率
ite=10000;lambda=1;eta=0.001

# パラメータを保存するデータ
alpha_beta_data=array(0,dim=c(ite,ncol(X)))

for(j in 1:ite){
  V=array(0,dim=c(nrow(X),nrow(X)))
  lambda=1/mean(((pis-exp(X%*%alpha))^2)/(pis*exp(X%*%alpha)^2))
  diag(V)=1/(exp(X%*%alpha)/lambda)
  D=diag(c(1/exp(X%*%alpha)))
  mat=solve(t(X)%*%V%*%(X))%*%t(X)%*%V%*%D%*%(pis-exp(X%*%alpha))
  alpha=alpha+eta*mat
  alpha_beta_data[j,]=alpha

  # 対数尤度
  loglik=sum(lambda*(-pis/(2*exp(X%*%alpha)^2)+1/exp(X%*%alpha))-(log(2*pi)+3*log(pis))/2-lambda/(2*pis)+log(lambda)/2)

  print(loglik)
}

predict=exp(X%*%alpha)

# 尤離度
d=lambda*sum((pis-exp(X%*%alpha))^2/(pis*exp(X%*%alpha)^2))

# 尤離度決定係数
R2_dev=1-d/(lambda*sum((pis-mean(pis))^2/(pis*mean(pis)^2)))

data=data.frame(X,pis,predict)

plot(c(1:nrow(data)),data$pis,type="p",col=2,xlim=c(1,nrow(data)),ylim=c(min(c(pis,predict)),max(c(pis,predict))),xlab="samples",ylab="Y",main="逆ガウス回帰model",pch=16)

par(new=T)

plot(c(1:nrow(data)),data$predict,type="p",col=3,xlim=c(1,nrow(data)),ylim=c(min(c(pis,predict)),max(c(pis,predict))),xlab="samples",ylab="Y",pch=1)
