# 計算例

# 分散共分散行列と平均・標準偏差・分散
data=data.frame(list=c("x1(身長)","x2(体重)","x3(胸囲)","x4(座高)"),average=c(149.06,40.26,73.98,80.02),variance=c(19.94,23.56,20.95,7.55),standard_deviation=c(4.47,4.85,4.58,2.75))

variance_covariance_matrix=matrix(c(19.94,10.5,6.59,8.63,10.5,23.56,19.71,7.97,6.59,19.71,20.95,3.93,8.63,7.97,3.93,7.55),ncol=4,nrow=4)

# 固有値、固有ベクトル
eigen_value=eigen(variance_covariance_matrix)$values
eigen_vector=eigen(variance_covariance_matrix)$vectors

# 累積寄与率と寄与率
propotion=eigen_value/sum(diag(variance_covariance_matrix))
accumulated_propotion=cumsum(propotion)

cov_mat=variance_covariance_matrix

# iterationによるラグランジュ関数の最適化による例

# 1回目（第1固有値、第1固有ベクトル）
# Broyden Quasi-Newton Algorithm
f<-function(x){
  lam=x[1];x=t(t(c(x[-1])))
  f1=2*cov_mat%*%x-2*lam*x
  f2=1-t(x)%*%x
  return(c(f1,f2))
}

X=rep(3,ncol(cov_mat)+1)
ite=10000;eta=0.01
H=diag(f(X))

for(l in 1:ite){
  if(sum(abs(f(X)))>10^(-9)){
    X_pre=X
    X=X-eta*H%*%f(X)
    s=X-X_pre
    y=f(X)-f(X_pre)
    H=H+((s-H%*%y)/as.numeric(t(s)%*%H%*%y))%*%t(s)%*%H
    print((f(X)))
  }
}

# 第1固有値
lam1=X[1]

# 第1固有ベクトル
w1=X[-1]

# 2回目（第2固有値、第2固有ベクトル）
# Broyden Quasi-Newton Algorithm
f<-function(x){
  lam=x[1];r=x[2];x=t(t(c(x[-c(1:2)])))
  f3=2*cov_mat%*%x-2*lam*x+r*w1
  f1=1-t(x)%*%x
  f2=sum(w1*x)
  return(c(f1,f2,f3))
}

X=rep(2.5,ncol(cov_mat)+2)
ite=100000;eta=0.01
H=diag(f(X))

for(l in 1:ite){
  if(sum(abs(f(X)))>10^(-9)){
    X_pre=X
    X=X-eta*H%*%f(X)
    s=X-X_pre
    y=f(X)-f(X_pre)
    H=H+((s-H%*%y)/as.numeric(t(s)%*%H%*%y))%*%t(s)%*%H
    print(sum(abs(f(X))))
  }
}

# 第2固有値
lam2=X[1]

# 第2固有ベクトル
w2=X[-c(1:2)]
